#ifndef __creaImageIOTreeDescriptor_h_INCLUDED__
#define __creaImageIOTreeDescriptor_h_INCLUDED__

#include <creaImageIOTreeAttributeMapType.h>
#include <creaImageIOTreeLevelDescriptor.h>


namespace creaImageIO
{
	using namespace std;
  namespace tree
  {
    
    /**
     * \ingroup Tree
     */
    
	///Descriptor of the structure of a Tree (number of levels, descriptors of each level, ...). 
	  //Any Tree has at least one level (level 0) of name "Root"
     
    class Descriptor
    {
    public:
      /// Ctor : creates the mandatory level 0 descriptor called "Root"
      Descriptor();
      /// Destructor
      ~Descriptor();

      /// Loads from a xml description file
      void LoadXML(const std::string& filename);
      /// Creates the default descriptor
      void CreateDefault();

      /// Returns the number of levels of the tree
      unsigned int GetNumberOfLevels() 
      { return mLevelDescriptorList.size(); }

      /// Returns the LevelDescriptor of a given level (const ref)
      const LevelDescriptor& GetLevelDescriptor(int level) const
      { return mLevelDescriptorList[level]; }

      /// Returns the AttributeDescriptorList of a given level (const ref)
      const LevelDescriptor::AttributeDescriptorListType& 
      GetAttributeDescriptorList(int level) const
      { return mLevelDescriptorList[level].GetAttributeDescriptorList(); }
  
      /// Adds a LevelDescriptor at the end of the list
      void Add(const LevelDescriptor&);
      
      /// Adds an AttributeDescriptor to level l
      void Add(const AttributeDescriptor&, int l);
      
      
      /// Builds the key to value map of all the attributes of the tree
	void BuildAttributeMap( AttributeMapType& ) const;

      /// The type of LevelDescriptor container
	typedef std::vector<LevelDescriptor> LevelDescriptorListType;
 
      /// Returns the list of tree levels (const)
	const LevelDescriptorListType& GetLevelDescriptorList() const { return mLevelDescriptorList; }

      /// Clears the Descriptor
      void Clear();

	  //Create Attribute Descriptors from a file
	void createDescriptorfromFile(const std::string &i_file);

    private:
      LevelDescriptorListType mLevelDescriptorList;
      /// Creates the mandatory level 0 descriptor called "Root"
      /// (assumes the list is empty)
      void CreateLevel0Descriptor();
  
    };
    // EO class Descriptor
    //=====================================================================
  } // EO namespace tree

} // EO namespace creaImageIO

// EOF
#endif  
