#ifndef __creaImageIOWxGimmickPanel_h_INCLUDED__
#define __creaImageIOWxGimmickPanel_h_INCLUDED__

#ifdef USE_WXWIDGETS

// Signal/slot mechanism for progress events
#include <boost/signal.hpp>
#include <boost/bind.hpp>

#include <creaImageIOWxGimmickView.h>
#include <creaWx.h>

namespace creaImageIO
{
  /**
   * \ingroup GUI
   */
  //=====================================================================
 //=====================================================================
  class CREAIMAGEIO_EXPORT WxGimmickPanel : public wxPanel
  {
  public:
    WxGimmickPanel();    
    WxGimmickPanel(wxWindow *parent, 
		   const wxWindowID id,
		   const wxPoint& pos, 
		   const wxSize& size,
		   int threads = 0);
    
    ~WxGimmickPanel();

      //=============================================
      typedef boost::signal<void (bool)> SendImageSignalType;
      typedef SendImageSignalType::slot_function_type SendImageCallbackType;
      //=============================================

     //==================================================================
      /// Adds the function f to the list of functions to call 
      /// when the addition progresses.
      /// f is of type ProgressCallbackType which is:
      /// void (*ProgressCallbackType)(Progress&)
      /// To pass a member function 'f' of an instance 'c' of a class 'C' 
      /// as callback you have to 'bind' it, i.e. call:
      /// ConnectSendImageObserver ( boost::bind( &C::f , c, _1 ) );
      void ConnectSendImageObserver(SendImageCallbackType callback);
     //==================================================================

	//===============================================================================================
	//Image Selection
	//===============================================================================================

	void GetSelectedImages(std::vector<vtkImageData*>& s, int dim)
    { mView->GetSelectedImages(s, dim); }

    void OnSelectedImage(bool t);

	void AddImagesToDB(std::string dir);

    //    DECLARE_EVENT_TABLE();
  private :
    
    Gimmick*       mGimmick;
    WxGimmickView* mView;

	///The sendImage signal
	SendImageSignalType mSendImageSignal;

  }; // class WxGimmickPanel
  //=====================================================================

  
} // EO namespace creaImageIO


#endif // USE_WIDGETS
// EOF
#endif  
