#include <creaImageIOWxGimmickReaderDialog.h>
#include <creaImageIOSystem.h>

namespace creaImageIO
{
  // CTor
  WxGimmickReaderDialog::WxGimmickReaderDialog(wxWindow *parent, 
					       wxWindowID id,
					       wxString title,
					       const wxPoint& pos,
					       const wxSize& size,
					       int min_dim,
						   int max_dim,
						   int output_dim,
					       int threads)
 :   wxDialog( parent, 
		  id, 
		  title,
		  pos,
		  size,
		  wxRESIZE_BORDER | 
		  wxSYSTEM_MENU  |
		  wxCLOSE_BOX |
		  wxMAXIMIZE_BOX | 
		  wxMINIMIZE_BOX | 
		  wxCAPTION  
	       ),
     mGimmick(0),
     mView(0)
  {
    GimmickDebugMessage(1,"WxGimmickReaderDialog::WxGimmickReaderDialog"
			<<std::endl);
    wxBoxSizer *topsizer = new wxBoxSizer(wxVERTICAL);
    
    try {
      
      mGimmick = new Gimmick();
      mGimmick->Initialize();
    
      
      mView = new WxGimmickView(mGimmick,
				this,
				TVID,
				wxDefaultPosition,
				size,
				min_dim,
				max_dim,
				threads);
      mView->Initialize();
	   // Connect the AddProgress callback
      mView->ConnectValidationObserver
      ( boost::bind( &WxGimmickReaderDialog::OnValid , this, _1 ) );
    }
    catch (crea::Exception e)
    {
      e.Print();
      return;
    }

    topsizer->Add( mView,1,wxGROW,0);

    wxSizer* bsizer = CreateSeparatedButtonSizer(wxOK|wxCANCEL);
    mOkButton = (wxButton*)FindWindowById(GetAffirmativeId(), this);
    mCancelButton = (wxButton*)FindWindowById(GetEscapeId(), this);
    
    mOkButton->Enable(false);

    topsizer->Add ( bsizer, 0, wxGROW );

    SetSizer( topsizer );     
    Layout(); 
  }

  /// Destructor
  WxGimmickReaderDialog::~WxGimmickReaderDialog()
  {
    GimmickDebugMessage(1,"WxGimmickReaderDialog::~WxGimmickReaderDialog"
			<<std::endl);
    if (mView) 
      {
		delete mView;
      }
    if (mGimmick) 
      {
	mGimmick->Finalize();
	delete mGimmick;
      }

  }

  ///Callback method on a selection
  void WxGimmickReaderDialog::OnValid(bool t)
  {
    mOkButton->Enable(t);
  }



 
  //================================================================
  //BEGIN_EVENT_TABLE(WxGimmickReaderDialog, wxDialog)
  //END_EVENT_TABLE()
  //================================================================


} // EO namespace creaImageIO


