#include <creaImageIOWxGimmickTools.h>

namespace creaImageIO
{
	/**
	** Begin of the threshold panel
	**/
	WxGimmickTools::WxGimmickTools(wxWindow* parent, wxString mCurrentDirectory)
	: wxPanel(parent, -1, wxDefaultPosition, wxSize(300,250), wxBORDER_SUNKEN)
	{
		_currentDir = mCurrentDirectory;
		_addFiles = true;
		
		_inputPath = new wxTextCtrl(this, wxID_ANY, _T(""), wxDefaultPosition, wxSize(150,30));
		_outputPath = new wxTextCtrl(this, wxID_ANY, _T(""), wxDefaultPosition, wxSize(150,30));
		_addCheckBox = new wxCheckBox(this, -1, _T("Add Images to Database?") );
		_addCheckBox->SetValue(_addFiles);

		wxButton *inputDir = new wxButton(this,wxID_ANY,_T("Input Directory"), wxDefaultPosition, wxSize(150,30) );
		wxButton *outputDir = new wxButton(this,wxID_ANY,_T("Output Directory"), wxDefaultPosition, wxSize(150,30) );

		
		Connect( inputDir->GetId(), wxEVT_COMMAND_BUTTON_CLICKED , (wxObjectEventFunction) &WxGimmickTools::onInputDir ); 
		Connect( outputDir->GetId(), wxEVT_COMMAND_BUTTON_CLICKED , (wxObjectEventFunction) &WxGimmickTools::onOutputDir );
		Connect( _addCheckBox->GetId(), wxEVT_COMMAND_CHECKBOX_CLICKED, (wxObjectEventFunction) &WxGimmickTools::onAddToDatabase );

		wxFlexGridSizer * textSizer	= new wxFlexGridSizer(1,2);
		textSizer -> Add( new wxStaticText(this,-1,_T("Transform a Bruker image into Dicom format."))  , 1, wxGROW );
		textSizer -> Add( new wxStaticText(this,-1,_T("If checkbox is selected images will be loaded into the DB."))  , 1, wxGROW );

		wxFlexGridSizer * sizer	= new wxFlexGridSizer(2,5);
		sizer -> Add( new wxStaticText(this,-1,_T(" "))  , 1, wxGROW );
		sizer -> Add( new wxStaticText(this,-1,_T(" "))  , 1, wxGROW );
		sizer -> Add( _inputPath, 1, wxGROW );
		sizer -> Add( inputDir, 1, wxGROW );
		sizer -> Add( _outputPath, 1, wxGROW );
		sizer -> Add( outputDir, 1, wxGROW );
		sizer -> Add( new wxStaticText(this,-1,_T(" "))  , 1, wxGROW );
		sizer -> Add( new wxStaticText(this,-1,_T(" "))  , 1, wxGROW );
		sizer -> Add( _addCheckBox, 1, wxGROW );

		wxFlexGridSizer * topSizer	= new wxFlexGridSizer(1,2);
		topSizer -> Add( textSizer, 1, wxGROW );
		topSizer -> Add( sizer, 1, wxGROW );
		this->SetSizer( topSizer );
		this->SetAutoLayout( true );
		this->Layout();
	}

	WxGimmickTools::~WxGimmickTools(){
	}

	wxString WxGimmickTools::getInputDir()
	{
		return _inputPath->GetLabel();
	}
		
	wxString WxGimmickTools::getOutputDir()
	{
		return _outputPath->GetLabel();
	}
		
	bool WxGimmickTools::getCheckBoxValue()
	{
		return _addCheckBox->GetValue();
	}
		
	void WxGimmickTools::onInputDir(wxCommandEvent& event)
	{
		long style = wxDD_DEFAULT_STYLE | wxDD_DIR_MUST_EXIST;
		wxDirDialog* FD = new wxDirDialog( 0, _T("Select the Input Directory"), _currentDir, style);
	    
		if (FD->ShowModal()==wxID_OK)
		{
			_inputPath->SetLabel(FD->GetPath());
		}
	}

	void WxGimmickTools::onOutputDir(wxCommandEvent& event)
	{
		long style = wxDD_DEFAULT_STYLE;
		wxDirDialog* FD = new wxDirDialog( 0, _T("Select the Output Directory"), _currentDir, style);
	    
		if (FD->ShowModal()==wxID_OK)
		{
			_outputPath->SetLabel(FD->GetPath());
		}
	}

	void WxGimmickTools::onAddToDatabase(wxCommandEvent& event)
	{
		_addFiles = _addCheckBox->GetValue();
	}

} // EO namespace creaImageIO


